// Copyright Notice
// ================
// BOCHS is Copyright 1994-1998 by Kevin P. Lawton.
//
// This file (MPU401.H) Copyright 1998 by Josef Drexler
// 
// BOCHS is commercial software.
// 
// For more information, read the file 'LICENSE' included in the bochs
// distribution.  If you don't have access to this file, or have questions
// regarding the licensing policy, the author may be contacted via:
// 
//     US Mail:  Kevin Lawton
//               528 Lexington St.
//               Waltham, MA 02452-3028
// 
//     EMail:    bochs@world.std.com


#if BX_USE_MPU401_SMF
#  define BX_MPU401_SMF  static
#  define BX_MPU401_THIS bx_mpu401.
#else
#  define BX_MPU401_SMF
#  define BX_MPU401_THIS this->
#endif

#define LOGFILE BX_MPU401_THIS s.logfile
#define DATAFILE BX_MPU401_THIS s.datafile
#define INBUF BX_MPU401_THIS s.inbuffer.
#define OUTBUF BX_MPU401_THIS s.outbuffer.

#define DATAPORT	(BX_MPU401_IO)
#define COMDPORT	(BX_MPU401_IO+1)
#define STATPORT	(BX_MPU401_IO+1)

#define OUTPUT_READY    0x40
#define INPUT_AVAIL     0x80
#define MPU_ACK         0xFE

#define OUTBUFLEN	16
#define INBUFLEN	64

// This is the class for the input and
// output buffers of the MPU401
class bx_mpu401_buffer {
public:

  bx_mpu401_buffer(void);
  ~bx_mpu401_buffer(void);
  void init(int bufferlen);

  Boolean put(Bit8u data);
  Boolean get(Bit8u *data);
  Boolean full(void);
  Boolean empty(void);

private:
  Bit8u *buffer;
  int head,tail,length;
  };

// The actual emulator class. It intercepts accesses to the
// MPU401 IO ports and writes the raw midi stream to /dev/midi00
class bx_mpu401_c {
public:

  bx_mpu401_c(void);
  ~bx_mpu401_c(void);
  BX_MPU401_SMF void   init(bx_devices_c *);

private:

  struct {
    FILE *logfile;
    FILE *datafile;
    Boolean uartmode;
    bx_mpu401_buffer outbuffer, inbuffer;
    } s;

  bx_devices_c *devices;

  static void   writelog(const char *str, ...);
  static void   writedata(const Bit8u value);

  static Bit32u read_handler(void *this_ptr, Bit32u address, unsigned io_len);
  static void   write_handler(void *this_ptr, Bit32u address, Bit32u value, unsigned io_len);
#if !BX_USE_MPU401_SMF
  Bit32u read(Bit32u address, unsigned io_len);
  void   write(Bit32u address, Bit32u value, unsigned io_len);
#endif
  };


#if BX_USE_MPU401_SMF
extern bx_mpu401_c bx_mpu401;
#endif
